/*

This code takes in an amplified audio signal through A0 pin
Applies FFT Algorithm to determine the magnitude of frequencies present
Uses that data to generate PWM Wave that is sent to a MOSFET which in turn controls the 12V RGB LED Strip

Written by Tarun R
*/
#include "fix_fft.h"                                  // library to perfom the fixed-point in-place Fast Fourier Transform

// if FFT_N=128, (9615 /2)/(128/2) = 75 hz per bin
// if FFT_N=256, (9615/2)/(266/2) = 37.6 hz per bin
const int FFT_N =128;                                 // The Arduino Uno does not have enough memory to support 256 samples.
                                                      // many samples you will get a larger resolution for the results.
                                                      // set FFT_N to 256 by using an Arduino Mega 2560.
char im[FFT_N], data[FFT_N];                          // variables for the FFT 
int i = 0, val,fall;
int R,R_data,R_avg,G,G_data,G_avg,B,B_data,B_avg;     // Variables to store LED PWM data                               


void setup()
{
  Serial.begin(9600);                                 // serial comms for debuging
  analogReference(DEFAULT);                           // Use default (5v) aref voltage.
};

void loop()
{
  int mini=1024, maxi=0;                              // set minumum & maximum ADC values
  for (i = 0; i < FFT_N; i++) {                       // take 128 or 256  point FFT samples
    val = analogRead(A0);                             // get audio from Analog 0
    Serial.print(A0);
    Serial.print("\n");
    data[i] = val / 4 - 128;                          // the reason it is divided by 4 and subtracted by 128 is because the "data" is of type "char" which is an 8 bit variable (-128 to 127). 
                                                      // So because analogRead returns 0-1023, dividing it by 4 gives you 0-255, subtracting it by 128 gives you -128-127.
                                                      // scale from 10 bit (0-1023) to 8 bit (0-255)
    im[i] = 0;                                        // No imaginary values are fed into the FFT algorithm
    if(val>maxi) maxi=val;                            // capture maximum level
    if(val<mini) mini=val;                            // capture minimum level
  };

 
  /*
   Performs foward or inverse Fourier transform.
   // fix_fft (char fr[], char fi[], int m, int inverse)
   // fr is a real data set,
   // fi is the imaginary data set, im at 0 here
   // the 3rd parameter m, then there are 2 ^ m 'bins'; if m = 7 then so 128; only the first half (64) contain the usable values
   // m is log2(n) where n (FFT_N) is number of data points (log2 (128) = 7), (log2 (256) = 8) 
   // 0 is set for forward transform. 1 would be inverse transform. Apparently inverse does not work,
*/
    
  //fix_fft(data, im, 8, 0);                          // perform the FFT on data for log2 (256) = 8 
  fix_fft(data, im, 7, 0);                            // perform the FFT on data for log2 (128) = 7


      // result array is a bin. 
      // interested in the absolute value of the transformation
      // The number of bins you get is half the amount of samples spanning the frequency range from zero to half the sampling rate.
      // the first half of array elements contain the real values after fix_fft 
      // In the end we get N/2 bins, each covering a frequency range of sampling_rate/N Hz.
      R_data=0;
      R_avg=0;
      G_data=0;
      G_avg=0;
      B_data=0;
      B_avg=0;
      
  for (i = 1; i < FFT_N/2; i++) {                                     // full spectrum;  here 4.8kHz bandwith, has 128 bins each 37.5Hz
                                                                      // 1st bin = 37.5Hz, 2nd bin = 37.5*2 Hz  etc.
                                                                      // change for loop values for specific range of interested frequency
                                                                      // ex:  for (i = 1; i < 64; i++)    
    int dat = sqrt(data[i] * data[i] + im[i] * im[i]);                // Determine the magnitude of each frequency bin 
    //dat = (2-dat/60)*dat;                                           // multiplication factor makes it look better  
    
    
    if(i>=1 && i<=7)                                                  // Grouping bins for low frequency
    {
      R_data = R_data+dat;
      R_avg = R_data/7;                                               // Average data
      R = map(R_avg,0,37,0 ,255)*2.2;                                 // Map it to 255 with multiplication factor (change as per your need)
      analogWrite(9, R);
      //Serial.print(R);
      //Serial.print("\n");
    }
   
    if(i>=20 && i<=44)                                                // Grouping bins for Mid frequency
    {
      G_data = G_data+dat;
      G_avg = G_data/24;                                              // Average Data
      G =map(G_avg,0,35,0,400)*1.091*2;                               // Map it to 255 with multiplication factor (change as per your need)
      analogWrite(10, G);
      //Serial.print(G);
      //Serial.print("\n");
    }

    if(i>=48 && i<=60)
    {
      B_data = B_data+dat;
      B_avg = B_data/12;
      B = map(B_avg,0,40,0,400)*1.19*2;
      analogWrite(11, B);
      //Serial.print(B);
      //Serial.print("\n");
    }
  };               
}; 
